@@ -262,7 +262,7 @@ gcpApplication::gcpApplication ():
 		}
 
 		// Initialize types
-		Object::AddType ("molecule", CreateMolecule, MoleculeType);
+		Object::AddType ("molecule", ::CreateMolecule, MoleculeType);
 		Object::AddType ("reaction", CreateReaction, ReactionType);
 		Object::SetCreationLabel (ReactionType, _("Create a new reaction"));
 		ReactionStepType = Object::AddType ("reaction-step", CreateReactionStep);
@@ -338,6 +338,9 @@ gcpApplication::gcpApplication ():
 	GError *error = NULL;
 	GCU_GCONF_GET (ROOTDIR"compression", int, CompressionLevel, 0)
 	GCU_GCONF_GET (ROOTDIR"tearable-mendeleiev", bool, TearableMendeleiev, false)
+	bool CopyAsText;
+	GCU_GCONF_GET (ROOTDIR"copy-as-text", bool, CopyAsText, false)
+	ClipboardFormats = CopyAsText? GCP_CLIPBOARD_ALL: GCP_CLIPBOARD_NO_TEXT;
 	m_NotificationId = gconf_client_notify_add (m_ConfClient, "/apps/gchempaint/settings", (GConfClientNotifyFunc) on_config_changed, this, NULL, NULL);
 	// make themes permanent with this as a dummy client
 	list <string> Names = ThemeManager.GetThemesNames ();
@@ -444,8 +447,16 @@ bool gcpApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 			file_type = SVG;
 		else if (!strcmp (mime_type, "image/x-eps"))
 			file_type = EPS;
-		else if ((pixbuf_type = GetPixbufTypeName (filename2, mime_type)))
+		else if ((pixbuf_type = GetPixbufTypeName (filename2, mime_type))) {
 			file_type = PIXBUF;
+			if (!ext) {
+				filename = filename2.c_str ();
+				i = strlen (filename) - 1;
+				while ((i > 0) && (filename[i] != '.') && (filename[i] != '/')) i--;
+				if (filename[i] == '/') i = 0;
+				ext = (i > 0)? filename + i + 1: NULL;
+			}
+		}
 	}
 	if (file_type < 0 || (!bSave && (file_type > CHEMISTRY))) {
 		GtkWidget* message = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
@@ -987,7 +998,8 @@ void gcpApplication::OnConfigChanged (GConfClient *client, guint cnxn_id, GConfE
 		gcpTools *Tools = dynamic_cast<gcpTools*> (GetDialog ("tools"));
 		if (Tools)
 			Tools->Update ();
-	}
+	} else 	if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"copy-as-text"))
+		ClipboardFormats = (gconf_value_get_bool (gconf_entry_get_value (entry)))? GCP_CLIPBOARD_ALL: GCP_CLIPBOARD_NO_TEXT;
 }
 
 void gcpApplication::TestSupportedType (char const *mime_type)
