@@ -863,19 +863,23 @@ bool gcpBond::BuildContextualMenu (GtkUIManager *UIManager, Object *object, doub
 	GtkAction *action;
 	action = gtk_action_new ("Bond", _("Bond"), NULL, NULL);
 	gtk_action_group_add_action (group, action);
+	g_object_unref (action);
 	if (is_before) {
 		action = gtk_action_new ("MoveBack", _("Move to back"), NULL, NULL);
 		g_signal_connect_swapped (action, "activate", G_CALLBACK (on_move_to_back), this);
 		gtk_action_group_add_action (group, action);
+		g_object_unref (action);
 		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Bond'><menuitem action='MoveBack'/></menu></popup></ui>", -1, NULL);
 	}
 	if (is_after) {
 		action = gtk_action_new ("BringFront", _("Bring to front"), NULL, NULL);
 		g_signal_connect_swapped (action, "activate", G_CALLBACK (on_bring_to_front), this);
 		gtk_action_group_add_action (group, action);
+		g_object_unref (action);
 		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Bond'><menuitem action='BringFront'/></menu></popup></ui>", -1, NULL);
 	}
 	gtk_ui_manager_insert_action_group (UIManager, group, 0);
+	g_object_unref (group);
 	if (!pAtom)
 		GetParent ()->BuildContextualMenu (UIManager, object, x, y);
 	return true;
