@@ -4,7 +4,7 @@
  * GChemPaint library
  * view.cc
  *
- * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -36,13 +36,18 @@
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
-#include <math.h>
 #include <unistd.h>
 #include <pango/pango-context.h>
 #include <gdk/gdkkeysyms.h>
+#include <glib/gi18n-lib.h>
 #include <libgnomeprint/gnome-print-job.h>
-#include <locale.h>
-#include <fstream>
+#include <cmath>
+#include <clocale>
+#ifdef HAVE_FSTREAM
+#	include <fstream>
+#else
+#	include <fstream.h>
+#endif
 
 /*
 Derivation of a new widget from gnome_canvas with an event for updating canvas size
@@ -193,11 +198,13 @@ gcpView::gcpView (gcpDocument *pDoc, bool Embedded)
 	m_UIManager = gtk_ui_manager_new ();
 	m_Dragging = false;
 	m_pWidget = NULL;
+	m_PangoContext = NULL;
 }
 
 gcpView::~gcpView ()
 {
-	g_object_unref (G_OBJECT (m_PangoContext));
+	if (m_PangoContext)
+		g_object_unref (G_OBJECT (m_PangoContext));
 	if (m_sFontName)
 		g_free (m_sFontName);
 	if (m_sSmallFontName)
@@ -281,7 +288,7 @@ bool gcpView::OnEvent(GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 			 	m_UIManager = gtk_ui_manager_new ();
 				result = pActiveTool->OnRightButtonClicked(this, Obj, event->button.x, event->button.y, m_UIManager);
 				if (Obj)
-					result |= Obj->BuildContextualMenu (m_UIManager, Obj);
+					result |= Obj->BuildContextualMenu (m_UIManager, Obj, x / GetZoomFactor (), y / GetZoomFactor ());
 				if (result) {
 					GtkWidget *w = gtk_ui_manager_get_widget (m_UIManager, "/popup");
 					gtk_menu_popup (GTK_MENU (w), NULL, NULL, NULL, NULL, 3,  gtk_get_current_event_time ());
@@ -358,8 +365,12 @@ GtkWidget* gcpView::CreateNewWidget()
 		if (pWidget)
 		{
 			gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(pWidget), "data");
-			std::map<Object*, GnomeCanvasGroup*>::iterator i;
-			for (i = pData->Items.begin(); i != pData->Items.end(); i++)
+			std::map<Object*, GnomeCanvasGroup*>::iterator i, iend = pData->Items.end();
+			for (i = pData->Items.begin(); i != iend; i++)
+				if ((*i).first->GetType () != BondType)
+				 (*i).first->Add(m_pWidget);
+			for (i = pData->Items.begin(); i != iend; i++)
+				if ((*i).first->GetType () == BondType)
 				 (*i).first->Add(m_pWidget);
 		}
 		else
@@ -382,11 +393,15 @@ void gcpView::OnDestroy(GtkWidget* widget)
 	else delete m_pDoc;
 }
 
-GnomeCanvasItem* gcpView::GetCanvasItem(GtkWidget* widget, Object* Object)
+GnomeCanvasItem* gcpView::GetCanvasItem (GtkWidget* widget, Object* Object)
 {
-	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(widget), "data");
-	if ((!pData) || (pData->View != this)) return NULL;
-	return (GnomeCanvasItem*) pData->Items[Object];
+	gcpWidgetData* pData = reinterpret_cast<gcpWidgetData*> (g_object_get_data (G_OBJECT (widget), "data"));
+	if ((!pData) || (pData->View != this))
+		return NULL;
+	GnomeCanvasItem *result = reinterpret_cast<GnomeCanvasItem*> (pData->Items[Object]);
+	if (!result)
+		pData->Items.erase (Object);
+	return result;
 }
 
 void gcpView::Print(GnomePrintContext *pc, gdouble width, gdouble height)
@@ -463,16 +478,6 @@ void gcpView::Remove(Object* pObject)
 	}
 }
 
-void gcpView::UpdateLabel(const gchar* label)
-{
-	GtkLabel* pLabel = (GtkLabel*)g_object_get_data(G_OBJECT(m_pWidget), "label");
-	if (pLabel) gtk_label_set_text(pLabel, label);
-	GtkContainer* pMenu = (GtkContainer*)g_object_get_data(G_OBJECT(m_pWidget), "menu");
-	if (!pMenu) return;
-	GList* l = gtk_container_get_children(pMenu);
-	if (GTK_IS_LABEL(l->data)) gtk_label_set_text((GtkLabel*)(l->data), label);
-}
-
 void gcpView::OnDeleteSelection (GtkWidget* w)
 {
 	m_pWidget = w;
@@ -534,8 +539,9 @@ GtkTargetEntry const targets[] = {
 	{(char *) "image/svg+xml",  0, 2},
 	{(char *) "image/png",  0, 3},
 	{(char *) "image/jpeg",  0, 4},
-	{(char *) "UTF8_STRING", 0, 1},
-	{(char *) "STRING", 0, 2}
+	{(char *) "image/bmp",  0, 5},
+	{(char *) "UTF8_STRING", 0, 6},
+	{(char *) "STRING", 0, 7}
 };
 
 void gcpView::OnCopySelection(GtkWidget* w, GtkClipboard* clipboard)
@@ -566,30 +572,28 @@ void gcpView::OnReceive(GtkClipboard* clipboard, GtkSelectionData* selection_dat
 	xmlDocPtr xml;
 	m_pData->UnselectAll();
 	switch (*DataType) {
-		case 0:
+		case GCP_CLIPBOARD_NATIVE:
 			xml = xmlParseMemory ((const char*) selection_data->data, selection_data->length);
 			m_pDoc->AddData (xml->children->children);
 			xmlFreeDoc (xml);
 			break;
-		case 5: {
+		case GCP_CLIPBOARD_UTF8_STRING: {
 				gcpText* text = new gcpText ();
-				PangoLayout *layout = text->GetLayout ();
-				pango_layout_set_text (layout, (const char*) selection_data->data, selection_data->length);
+				text->SetText ((char const*) selection_data->data);
 				text->OnChanged (true);
 				m_pDoc->AddObject (text);
 				m_pData->SetSelected (text);
 			}
 			break;
-		case 6: {
+		case GCP_CLIPBOARD_STRING: {
 				gcpText* text = new gcpText ();
-				PangoLayout *layout = text->GetLayout ();
 				if (!g_utf8_validate ((const char*) selection_data->data, selection_data->length, NULL)) {
 					gsize r, w;
 					gchar* newstr = g_locale_to_utf8 ((const char*) selection_data->data, selection_data->length, &r, &w, NULL);
-					pango_layout_set_text (layout, newstr, w);
+					text->SetText (newstr);
 					g_free (newstr);
 				} else
-					pango_layout_set_text (layout, (const char*) selection_data->data, selection_data->length);
+					text->SetText ((char const*) selection_data->data);
 				text->OnChanged (true);
 				m_pDoc->AddObject (text);
 				m_pData->SetSelected (text);
@@ -769,6 +773,22 @@ void gcpView::OnSelectAll()
 		pActiveTool->AddSelection(m_pData);
 }
 
+static gboolean do_save_image (const gchar *buf, gsize count, GError **error, gpointer data)
+{
+	GnomeVFSHandle *handle = (GnomeVFSHandle*) data;
+	GnomeVFSFileSize written = 0;
+	GnomeVFSResult res;
+	while (count) {
+		res = gnome_vfs_write (handle, buf, count, &written);
+		if (res != GNOME_VFS_OK) {
+			g_set_error (error, g_quark_from_static_string ("gchempaint"), res, gnome_vfs_result_to_string (res));
+			return false;
+		}
+		count -= written;
+	}
+	return true;
+}
+
 void gcpView::ExportImage (string const &filename, const char* type, int resolution)
 {
 	ArtDRect rect;
@@ -957,32 +977,18 @@ void gcpView::ExportImage (string const &filename, const char* type, int resolut
 		xmlSaveFormatFile (filename.c_str (), doc, true);
 		xmlFreeDoc (doc);
 	} else {
-		double zoom;
-		if (resolution > 0) {
-			int screenres = m_pDoc->GetApp ()->GetScreenResolution ();
-			zoom = (double) resolution / screenres;
-			w = (int) rint ((double) w * zoom);
-			h = (int) rint ((double) h * zoom);
-		} else
-		zoom = 1.;
-		gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), zoom);
-		gnome_canvas_update_now (GNOME_CANVAS (m_pWidget));
-		GdkPixbuf *pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, w, h);
-		gdk_pixbuf_fill (pixbuf, 0xffffffff);
-		GnomeCanvasBuf buf;
-		buf.buf = gdk_pixbuf_get_pixels (pixbuf);
-		buf.rect.x0 = (int) floor (rect.x0 * zoom);
-		buf.rect.x1 = (int) ceil (rect.x1 * zoom);
-		buf.rect.y0 = (int) floor (rect.y0 * zoom);
-		buf.rect.y1 = (int) ceil (rect.y1 * zoom);
-		buf.buf_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-		buf.bg_color = 0xffffff;
-		buf.is_buf = 1;
-		(* GNOME_CANVAS_ITEM_GET_CLASS (m_pData->Group)->render) (GNOME_CANVAS_ITEM (m_pData->Group), &buf);
-		gdk_pixbuf_save (pixbuf, filename.c_str (), type, NULL, NULL);
+		GdkPixbuf *pixbuf = BuildPixbuf (resolution);
+		GnomeVFSHandle *handle = NULL;
+		if (gnome_vfs_create (&handle, filename.c_str (), GNOME_VFS_OPEN_WRITE, true, 0644) == GNOME_VFS_OK) {
+			GError *error = NULL;
+			gdk_pixbuf_save_to_callbackv (pixbuf, do_save_image, handle, type, NULL, NULL, &error);
+			if (error) {
+				cerr << _("Unable to save image file: ") << error->message << endl;
+				g_error_free (error);
+			}
+			gnome_vfs_close (handle); // hope there will be no error there
+		}
 		g_object_unref (pixbuf);
-		// restore zoom level
-		gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), m_pData->Zoom);
 	}
 	m_pData->ShowSelection (true);
 }
@@ -1037,11 +1043,42 @@ xmlDocPtr gcpView::BuildSVG ()
 	return doc;
 }
 
+GdkPixbuf *gcpView::BuildPixbuf (int resolution)
+{
+	ArtDRect rect;
+	m_pData->GetObjectBounds (m_pDoc, &rect);
+	m_pData->ShowSelection (false);
+	int w = (int) (ceil (rect.x1) - floor (rect.x0)), h = (int) (ceil (rect.y1) - floor (rect.y0));
+	double zoom;
+	if (resolution > 0) {
+		int screenres = m_pDoc->GetApp ()->GetScreenResolution ();
+		zoom = (double) resolution / screenres;
+		w = (int) rint ((double) w * zoom);
+		h = (int) rint ((double) h * zoom);
+	} else
+		zoom = 1.;
+	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), zoom);
+	gnome_canvas_update_now (GNOME_CANVAS (m_pWidget));
+	GdkPixbuf *pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, w, h);
+	gdk_pixbuf_fill (pixbuf, 0xffffffff);
+	GnomeCanvasBuf buf;
+	buf.buf = gdk_pixbuf_get_pixels (pixbuf);
+	buf.rect.x0 = (int) floor (rect.x0 * zoom);
+	buf.rect.x1 = (int) ceil (rect.x1 * zoom);
+	buf.rect.y0 = (int) floor (rect.y0 * zoom);
+	buf.rect.y1 = (int) ceil (rect.y1 * zoom);
+	buf.buf_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+	buf.bg_color = 0xffffff;
+	buf.is_buf = 1;
+	(* GNOME_CANVAS_ITEM_GET_CLASS (m_pData->Group)->render) (GNOME_CANVAS_ITEM (m_pData->Group), &buf);
+	// restore zoom level
+	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), m_pData->Zoom);
+	return pixbuf;
+}
+
 void gcpView::EnsureSize ()
 {
 	GnomeCanvas *canvas = GNOME_CANVAS (m_pWidget);
-	while (canvas->idle_id)
-		gtk_main_iteration();
 	gnome_canvas_update_now (canvas);
 	if (GTK_WIDGET_REALIZED (m_pWidget))
 		g_signal_emit_by_name (m_pWidget, "update_bounds");
@@ -1052,6 +1089,8 @@ void gcpView::Zoom (double zoom)
 	m_pData->Zoom = zoom;
 	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), zoom);
 	EnsureSize ();
+	// Call OnSize to be certain that the canvas scroll region will be correct
+	OnSize (m_pWidget, m_width, m_height);
 }
 
 void gcpView::ShowCursor (bool show)
@@ -1059,3 +1098,31 @@ void gcpView::ShowCursor (bool show)
 	if (m_ActiveRichText)
 		g_object_set (G_OBJECT (m_ActiveRichText), "editing", show, NULL);
 }
+
+void gcpView::UpdateTheme ()
+{
+	if (m_sFontName)
+		g_free (m_sFontName);
+	if (m_sSmallFontName)
+		g_free (m_sSmallFontName);
+	pango_font_description_free (m_PangoFontDesc);
+	pango_font_description_free (m_PangoSmallFontDesc);
+	gcpTheme *pTheme = m_pDoc->GetTheme ();
+	m_PangoFontDesc = pango_font_description_new ();
+	pango_font_description_set_family (m_PangoFontDesc, pTheme->GetFontFamily ());
+	pango_font_description_set_style (m_PangoFontDesc, pTheme->GetFontStyle ());
+	pango_font_description_set_weight (m_PangoFontDesc, pTheme->GetFontWeight ());
+	pango_font_description_set_variant (m_PangoFontDesc, pTheme->GetFontVariant ());
+	pango_font_description_set_stretch (m_PangoFontDesc, pTheme->GetFontStretch ());
+	pango_font_description_set_size (m_PangoFontDesc, pTheme->GetFontSize ());
+	m_sFontName = pango_font_description_to_string (m_PangoFontDesc);
+	m_PangoSmallFontDesc = pango_font_description_new ();
+	pango_font_description_set_family (m_PangoSmallFontDesc, pTheme->GetFontFamily ());
+	pango_font_description_set_style (m_PangoSmallFontDesc, pTheme->GetFontStyle ());
+	pango_font_description_set_weight (m_PangoSmallFontDesc, pTheme->GetFontWeight ());
+	pango_font_description_set_variant (m_PangoSmallFontDesc, pTheme->GetFontVariant ());
+	pango_font_description_set_stretch (m_PangoSmallFontDesc, pTheme->GetFontStretch ());
+	pango_font_description_set_size (m_PangoSmallFontDesc, pTheme->GetFontSize () * 2 / 3);
+	m_sSmallFontName = pango_font_description_to_string (m_PangoSmallFontDesc);
+	Update (m_pDoc);
+}
